"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.findBackgroundColor = exports.parseColor = exports.locateFiles = exports.isError = exports.loadSVG = void 0;
const path_1 = __importDefault(require("path"));
const svg_js_1 = require("@svgdotjs/svg.js");
const debug_1 = __importDefault(require("debug"));
const expand_tilde_1 = __importDefault(require("expand-tilde"));
const fast_glob_1 = __importDefault(require("fast-glob"));
const fs_extra_1 = __importDefault(require("fs-extra"));
const sharp_1 = __importDefault(require("sharp"));
const debug = (0, debug_1.default)('sqip');
const loadSVG = async (svg) => {
    const { createSVGWindow } = await import('svgdom');
    const window = createSVGWindow();
    const document = window.document;
    (0, svg_js_1.registerWindow)(window, document);
    return { svg: (0, svg_js_1.SVG)(svg), SVG: svg_js_1.SVG };
};
exports.loadSVG = loadSVG;
function isError(error) {
    return error instanceof Error;
}
exports.isError = isError;
async function locateFiles(input) {
    const enhancedInput = (0, expand_tilde_1.default)(input);
    let globPattern = enhancedInput;
    try {
        const stat = await fs_extra_1.default.lstat(enhancedInput);
        if (stat.isFile()) {
            debug(`input ${input} is a file. Skip file search.`);
            return [enhancedInput];
        }
        if (stat.isDirectory()) {
            debug(`input ${input} is a directory. Enhancing with * to match all files.`);
            globPattern = `${path_1.default.resolve(enhancedInput)}/*`;
        }
    }
    catch (err) {
        if (isError(err) && err instanceof TypeError) {
            if (err.code === 'ENOENT') {
                throw err;
            }
        }
    }
    // Find all files matching the enhanced glob
    const files = await (0, fast_glob_1.default)(globPattern, {
        onlyFiles: true,
        extglob: true,
        absolute: true
    });
    // Test if files are found
    if (!files.length) {
        throw new Error(`Unable to find any files via ${globPattern}. Make sure the file exists.

If you are using globbing patterns, the following features are supported:

https://github.com/micromatch/micromatch#matching-features`);
    }
    return files;
}
exports.locateFiles = locateFiles;
function parseColor({ palette, color }) {
    // @todo test, fallback to or detect transparent as color (for bg)
    return palette[color]?.hex || color;
}
exports.parseColor = parseColor;
async function findBackgroundColor(imageBuffer) {
    const image = (0, sharp_1.default)(imageBuffer);
    const { width, height } = await image.metadata();
    if (!width || !height) {
        throw new Error('Unable to detect image sizes for background color extraction.');
    }
    // Sample edge pixels
    const edgePixels = await Promise.all([
        // Top left corner
        image
            .clone()
            .extract({ left: 0, top: 0, width: 1, height: 1 })
            .raw()
            .toBuffer(),
        // Top right corner
        image
            .clone()
            .extract({ left: 1, top: 1, width: 1, height: 1 })
            .raw()
            .toBuffer(),
        // Bottom left corner
        image
            .clone()
            .extract({ left: 0, top: height - 1, width: 1, height: 1 })
            .raw()
            .toBuffer(),
        // Bottom right corner
        image
            .clone()
            .extract({ left: width - 1, top: height - 1, width: 1, height: 1 })
            .raw()
            .toBuffer()
    ]);
    // Process edge pixels to find the most common color including alpha channel
    const colors = edgePixels.map((pixel) => {
        // Convert raw pixel data (RGBA) to hex color, including alpha
        return '#' + pixel.toString('hex');
    });
    // Count occurrences of each color, considering the alpha channel
    const colorCount = colors.reduce((acc, color) => {
        acc[color] = (acc[color] || 0) + 1;
        return acc;
    }, {});
    // Find the most frequent color, taking into account transparency
    const backgroundColor = Object.keys(colorCount).reduce((a, b) => colorCount[a] > colorCount[b] ? a : b);
    // If you want to detect and handle (semi) transparent backgrounds specifically
    const isTransparent = (color) => {
        // Extract the alpha value from the hex color
        const alpha = parseInt(color.slice(7, 9), 16) / 255;
        return alpha < 1;
    };
    // Example usage: Check if the background is transparent
    if (isTransparent(backgroundColor)) {
        return backgroundColor;
    }
    return backgroundColor;
}
exports.findBackgroundColor = findBackgroundColor;
//# sourceMappingURL=helpers.js.map